@@ -14,26 +14,22 @@
 # limitations under the License.
 
 """
-Amazon EC2 driver
+Amazon EC2, Eucalyptus and Nimbus drivers.
 """
+
 from __future__ import with_statement
 
 import sys
 import base64
-import hmac
 import os
-import time
 import copy
 
-from hashlib import sha256
 from xml.etree import ElementTree as ET
 
-from libcloud.utils.py3 import urlquote
 from libcloud.utils.py3 import b
 
 from libcloud.utils.xml import fixxpath, findtext, findattr, findall
-from libcloud.common.base import ConnectionUserAndKey
-from libcloud.common.aws import AWSBaseResponse
+from libcloud.common.aws import AWSBaseResponse, SignedAWSConnection
 from libcloud.common.types import (InvalidCredsError, MalformedResponseError,
                                    LibcloudError)
 from libcloud.compute.providers import Provider
@@ -41,23 +37,14 @@ from libcloud.compute.types import NodeState
 from libcloud.compute.base import Node, NodeDriver, NodeLocation, NodeSize
 from libcloud.compute.base import NodeImage, StorageVolume
 
-EC2_US_EAST_HOST = 'ec2.us-east-1.amazonaws.com'
-EC2_US_WEST_HOST = 'ec2.us-west-1.amazonaws.com'
-EC2_US_WEST_OREGON_HOST = 'ec2.us-west-2.amazonaws.com'
-EC2_EU_WEST_HOST = 'ec2.eu-west-1.amazonaws.com'
-EC2_AP_SOUTHEAST_HOST = 'ec2.ap-southeast-1.amazonaws.com'
-EC2_AP_NORTHEAST_HOST = 'ec2.ap-northeast-1.amazonaws.com'
-EC2_SA_EAST_HOST = 'ec2.sa-east-1.amazonaws.com'
-
 API_VERSION = '2010-08-31'
-
-NAMESPACE = "http://ec2.amazonaws.com/doc/%s/" % (API_VERSION)
+NAMESPACE = 'http://ec2.amazonaws.com/doc/%s/' % (API_VERSION)
 
 """
 Sizes must be hardcoded, because Amazon doesn't provide an API to fetch them.
 From http://aws.amazon.com/ec2/instance-types/
 """
-EC2_INSTANCE_TYPES = {
+INSTANCE_TYPES = {
     't1.micro': {
         'id': 't1.micro',
         'name': 'Micro Instance',
@@ -128,6 +115,20 @@ EC2_INSTANCE_TYPES = {
         'disk': 1690,
         'bandwidth': None
     },
+    'm3.xlarge': {
+        'id': 'm3.xlarge',
+        'name': 'Extra Large Instance',
+        'ram': 15360,
+        'disk': None,
+        'bandwidth': None
+    },
+    'm3.2xlarge': {
+        'id': 'm3.2xlarge',
+        'name': 'Double Extra Large Instance',
+        'ram': 30720,
+        'disk': None,
+        'bandwidth': None
+    },
     'cg1.4xlarge': {
         'id': 'cg1.4xlarge',
         'name': 'Cluster GPU Quadruple Extra Large Instance',
@@ -148,18 +149,194 @@ EC2_INSTANCE_TYPES = {
         'ram': 63488,
         'disk': 3370,
         'bandwidth': None
+    },
+    'cr1.8xlarge': {
+        'id': 'cr1.8xlarge',
+        'name': 'High Memory Cluster Eight Extra Large',
+        'ram': 244000,
+        'disk': 240,
+        'bandwidth': None
+    },
+    'hs1.8xlarge': {
+        'id': 'hs1.8xlarge',
+        'name': 'High Storage Eight Extra Large Instance',
+        'ram': 119808,
+        'disk': 48000,
+        'bandwidth': None
     }
 }
 
-CLUSTER_INSTANCES_IDS = ['cg1.4xlarge', 'cc1.4xlarge', 'cc2.8xlarge']
+REGION_DETAILS = {
+    'us-east-1': {
+        'endpoint': 'ec2.us-east-1.amazonaws.com',
+        'api_name': 'ec2_us_east',
+        'country': 'USA',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge',
+            'cc1.4xlarge',
+            'cc2.8xlarge',
+            'cg1.4xlarge',
+            'cr1.8xlarge',
+            'hs1.8xlarge'
+        ]
+    },
+    'us-west-1': {
+        'endpoint': 'ec2.us-west-1.amazonaws.com',
+        'api_name': 'ec2_us_west',
+        'country': 'USA',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge'
+        ]
+    },
+    'us-west-2': {
+        'endpoint': 'ec2.us-west-2.amazonaws.com',
+        'api_name': 'ec2_us_west_oregon',
+        'country': 'US',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'c1.medium',
+            'c1.xlarge',
+            'cc2.8xlarge'
+        ]
+    },
+    'eu-west-1': {
+        'endpoint': 'ec2.eu-west-1.amazonaws.com',
+        'api_name': 'ec2_eu_west',
+        'country': 'Ireland',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge',
+            'cc2.8xlarge'
+        ]
+    },
+    'ap-southeast-1': {
+        'endpoint': 'ec2.ap-southeast-1.amazonaws.com',
+        'api_name': 'ec2_ap_southeast',
+        'country': 'Singapore',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge'
+        ]
+    },
+    'ap-northeast-1': {
+        'endpoint': 'ec2.ap-northeast-1.amazonaws.com',
+        'api_name': 'ec2_ap_northeast',
+        'country': 'Japan',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge'
+        ]
+    },
+    'sa-east-1': {
+        'endpoint': 'ec2.sa-east-1.amazonaws.com',
+        'api_name': 'ec2_sa_east',
+        'country': 'Brazil',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'c1.medium',
+            'c1.xlarge'
 
-EC2_US_EAST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_US_WEST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_EU_WEST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_AP_SOUTHEAST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_AP_NORTHEAST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_US_WEST_OREGON_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
-EC2_SA_EAST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
+        ]
+    },
+    'ap-southeast-2': {
+        'endpoint': 'ec2.ap-southeast-2.amazonaws.com',
+        'api_name': 'ec2_ap_southeast_2',
+        'country': 'Australia',
+        'instance_types': [
+            't1.micro',
+            'm1.small',
+            'm1.medium',
+            'm1.large',
+            'm1.xlarge',
+            'm2.xlarge',
+            'm2.2xlarge',
+            'm2.4xlarge',
+            'm3.xlarge',
+            'm3.2xlarge',
+            'c1.medium',
+            'c1.xlarge'
+        ]
+    },
+    'nimbus': {
+        # Nimbus clouds have 3 EC2-style instance types but their particular
+        # RAM allocations are configured by the admin
+        'country': 'custom',
+        'instance_types': [
+            'm1.small',
+            'm1.large',
+            'm1.xlarge'
+        ]
+    }
+}
+
+VALID_EC2_DATACENTERS = REGION_DETAILS.keys()
+VALID_EC2_DATACENTERS = [d for d in VALID_EC2_DATACENTERS if d != 'nimbus']
 
 
 class EC2NodeLocation(NodeLocation):
@@ -209,57 +386,15 @@ class EC2Response(AWSBaseResponse):
         return "\n".join(err_list)
 
 
-class EC2Connection(ConnectionUserAndKey):
+class EC2Connection(SignedAWSConnection):
     """
-    Represents a single connection to the EC2 Endpoint
+    Represents a single connection to the EC2 Endpoint.
     """
 
-    host = EC2_US_EAST_HOST
+    version = API_VERSION
+    host = REGION_DETAILS['us-east-1']['endpoint']
     responseCls = EC2Response
 
-    def add_default_params(self, params):
-        params['SignatureVersion'] = '2'
-        params['SignatureMethod'] = 'HmacSHA256'
-        params['AWSAccessKeyId'] = self.user_id
-        params['Version'] = API_VERSION
-        params['Timestamp'] = time.strftime('%Y-%m-%dT%H:%M:%SZ',
-                                            time.gmtime())
-        params['Signature'] = self._get_aws_auth_param(params, self.key,
-                                                       self.action)
-        return params
-
-    def _get_aws_auth_param(self, params, secret_key, path='/'):
-        """
-        Creates the signature required for AWS, per
-        http://bit.ly/aR7GaQ [docs.amazonwebservices.com]:
-
-        StringToSign = HTTPVerb + "\n" +
-                       ValueOfHostHeaderInLowercase + "\n" +
-                       HTTPRequestURI + "\n" +
-                       CanonicalizedQueryString <from the preceding step>
-        """
-        keys = list(params.keys())
-        keys.sort()
-        pairs = []
-        for key in keys:
-            pairs.append(urlquote(key, safe='') + '=' +
-                         urlquote(params[key], safe='-_~'))
-
-        qs = '&'.join(pairs)
-
-        hostname = self.host
-        if (self.secure and self.port != 443) or \
-           (not self.secure and self.port != 80):
-            hostname += ":" + str(self.port)
-
-        string_to_sign = '\n'.join(('GET', hostname, path, qs))
-
-        b64_hmac = base64.b64encode(
-            hmac.new(b(secret_key), b(string_to_sign),
-                     digestmod=sha256).digest()
-        )
-        return b64_hmac.decode('utf-8')
-
 
 class ExEC2AvailabilityZone(object):
     """
@@ -279,28 +414,21 @@ class ExEC2AvailabilityZone(object):
                 % (self.name, self.zone_state, self.region_name))
 
 
-class EC2NodeDriver(NodeDriver):
+class BaseEC2NodeDriver(NodeDriver):
     """
-    Amazon EC2 node driver
+    Base Amazon EC2 node driver.
+
+    Used for main EC2 and other derivate driver classes to inherit from it.
     """
 
     connectionCls = EC2Connection
-    type = Provider.EC2
-    api_name = 'ec2_us_east'
-    name = 'Amazon EC2 (us-east-1)'
-    website = 'http://aws.amazon.com/ec2/'
-    friendly_name = 'Amazon US N. Virginia'
-    country = 'US'
-    region_name = 'us-east-1'
     path = '/'
-
-    _instance_types = EC2_US_EAST_INSTANCE_TYPES
     features = {'create_node': ['ssh_key']}
 
     NODE_STATE_MAP = {
         'pending': NodeState.PENDING,
         'running': NodeState.RUNNING,
-        'shutting-down': NodeState.TERMINATED,
+        'shutting-down': NodeState.UNKNOWN,
         'terminated': NodeState.TERMINATED
     }
 
@@ -505,30 +633,35 @@ class EC2NodeDriver(NodeDriver):
         return nodes
 
     def list_sizes(self, location=None):
-        # Cluster instances are not available in all the regions
-        if self.region_name == 'us-east-1':
-            ignored_size_ids = None
-        elif self.region_name == 'eu-west-1':
-            ignored_size_ids = CLUSTER_INSTANCES_IDS[:-1]
-        else:
-            ignored_size_ids = CLUSTER_INSTANCES_IDS
-
-        sizes = self._get_sizes(ignored_size_ids=ignored_size_ids)
-        return sizes
-
-    def _get_sizes(self, ignored_size_ids=None):
-        ignored_size_ids = ignored_size_ids or []
+        available_types = REGION_DETAILS[self.region_name]['instance_types']
         sizes = []
-        for key, values in self._instance_types.items():
-            if key in ignored_size_ids:
-                continue
-            attributes = copy.deepcopy(values)
-            attributes.update({'price': self._get_size_price(size_id=key)})
+
+        for instance_type in available_types:
+            attributes = INSTANCE_TYPES[instance_type]
+            attributes = copy.deepcopy(attributes)
+            price = self._get_size_price(size_id=instance_type)
+            attributes.update({'price': price})
             sizes.append(NodeSize(driver=self, **attributes))
         return sizes
 
-    def list_images(self, location=None):
+    def list_images(self, location=None, ex_image_ids=None):
+        """
+        List all images
+
+        Ex_image_ids parameter is used to filter the list of
+        images that should be returned. Only the images
+        with the corresponding image ids will be returned.
+
+        @param      ex_image_ids: List of C{NodeImage.id}
+        @type       ex_image_ids: C{list} of C{str}
+
+        @rtype: C{list} of L{NodeImage}
+        """
         params = {'Action': 'DescribeImages'}
+
+        if ex_image_ids:
+            params.update(self._pathlist('ImageId', ex_image_ids))
+
         images = self._to_images(
             self.connection.request(self.path, params=params).object
         )
@@ -536,10 +669,10 @@ class EC2NodeDriver(NodeDriver):
 
     def list_locations(self):
         locations = []
-        for index, availability_zone in\
+        for index, availability_zone in \
                 enumerate(self.ex_list_availability_zones()):
                     locations.append(EC2NodeLocation(
-                        index, self.friendly_name, self.country, self,
+                        index, availability_zone, self.country, self,
                         availability_zone)
                     )
         return locations
@@ -650,7 +783,7 @@ class EC2NodeDriver(NodeDriver):
 
         @note: This is a non-standard extension API, and only works for EC2.
 
-        @rtype: C{list}
+        @rtype: C{list} of C{str}
         """
 
         params = {
@@ -743,8 +876,8 @@ class EC2NodeDriver(NodeDriver):
         @param      from_port: The beginning of the port range to open
         @type       from_port: C{str}
 
-        @param      end_port: The end of the port range to open
-        @type       end_port: C{str}
+        @param      to_port: The end of the port range to open
+        @type       to_port: C{str}
 
         @param      cidr_ip: The ip to allow traffic for.
         @type       cidr_ip: C{str}
@@ -752,7 +885,7 @@ class EC2NodeDriver(NodeDriver):
         @param      protocol: tcp/udp/icmp
         @type       protocol: C{str}
 
-        @rtype: C{boolean}
+        @rtype: C{bool}
         """
 
         params = {'Action': 'AuthorizeSecurityGroupIngress',
@@ -762,7 +895,8 @@ class EC2NodeDriver(NodeDriver):
                   'ToPort': str(to_port),
                   'CidrIp': cidr_ip}
         try:
-            resp = self.connection.request(self.path, params=params.copy()).object
+            resp = self.connection.request(
+                self.path, params=params.copy()).object
             return bool(findtext(element=resp, xpath='return',
                                  namespace=NAMESPACE))
         except Exception:
@@ -1143,6 +1277,11 @@ class EC2NodeDriver(NodeDriver):
 
         @keyword    ex_clienttoken: Unique identifier to ensure idempotency
         @type       ex_clienttoken: C{str}
+
+        @keyword    ex_blockdevicemappings: C{list} of C{dict} block device
+                    mappings. Example:
+                    [{'DeviceName': '/dev/sdb', 'VirtualName': 'ephemeral0'}]
+        @type       ex_blockdevicemappings: C{list} of C{dict}
         """
         image = kwargs["image"]
         size = kwargs["size"]
@@ -1180,6 +1319,13 @@ class EC2NodeDriver(NodeDriver):
         if 'ex_clienttoken' in kwargs:
             params['ClientToken'] = kwargs['ex_clienttoken']
 
+        if 'ex_blockdevicemappings' in kwargs:
+            for index, mapping in enumerate(kwargs['ex_blockdevicemappings']):
+                params['BlockDeviceMapping.%d.DeviceName' % (index + 1)] = \
+                    mapping['DeviceName']
+                params['BlockDeviceMapping.%d.VirtualName' % (index + 1)] = \
+                    mapping['VirtualName']
+
         object = self.connection.request(self.path, params=params).object
         nodes = self._to_nodes(object, 'instancesSet/item')
 
@@ -1242,6 +1388,31 @@ class EC2NodeDriver(NodeDriver):
         return self._get_terminate_boolean(res)
 
 
+class EC2NodeDriver(BaseEC2NodeDriver):
+    """
+    Amazon EC2 node driver.
+    """
+
+    connectionCls = EC2Connection
+    type = Provider.EC2
+    name = 'Amazon EC2'
+    website = 'http://aws.amazon.com/ec2/'
+    path = '/'
+
+    region_name = 'us-east-1'
+    country = 'USA'
+    api_name = 'ec2_us_east'
+
+    features = {'create_node': ['ssh_key']}
+
+    NODE_STATE_MAP = {
+        'pending': NodeState.PENDING,
+        'running': NodeState.RUNNING,
+        'shutting-down': NodeState.UNKNOWN,
+        'terminated': NodeState.TERMINATED
+    }
+
+
 class IdempotentParamError(LibcloudError):
     """
     Request used the same client token as a previous,
@@ -1256,21 +1427,19 @@ class EC2EUConnection(EC2Connection):
     """
     Connection class for EC2 in the Western Europe Region
     """
-    host = EC2_EU_WEST_HOST
+    host = REGION_DETAILS['eu-west-1']['endpoint']
 
 
 class EC2EUNodeDriver(EC2NodeDriver):
     """
-    Driver class for EC2 in the Western Europe Region
+    Driver class for EC2 in the Western Europe Region.
     """
-
     api_name = 'ec2_eu_west'
     name = 'Amazon EC2 (eu-west-1)'
     friendly_name = 'Amazon Europe Ireland'
     country = 'IE'
     region_name = 'eu-west-1'
     connectionCls = EC2EUConnection
-    _instance_types = EC2_EU_WEST_INSTANCE_TYPES
 
 
 class EC2USWestConnection(EC2Connection):
@@ -1278,7 +1447,7 @@ class EC2USWestConnection(EC2Connection):
     Connection class for EC2 in the Western US Region
     """
 
-    host = EC2_US_WEST_HOST
+    host = REGION_DETAILS['us-west-1']['endpoint']
 
 
 class EC2USWestNodeDriver(EC2NodeDriver):
@@ -1292,7 +1461,6 @@ class EC2USWestNodeDriver(EC2NodeDriver):
     country = 'US'
     region_name = 'us-west-1'
     connectionCls = EC2USWestConnection
-    _instance_types = EC2_US_WEST_INSTANCE_TYPES
 
 
 class EC2USWestOregonConnection(EC2Connection):
@@ -1300,7 +1468,7 @@ class EC2USWestOregonConnection(EC2Connection):
     Connection class for EC2 in the Western US Region (Oregon).
     """
 
-    host = EC2_US_WEST_OREGON_HOST
+    host = REGION_DETAILS['us-west-2']['endpoint']
 
 
 class EC2USWestOregonNodeDriver(EC2NodeDriver):
@@ -1314,28 +1482,27 @@ class EC2USWestOregonNodeDriver(EC2NodeDriver):
     country = 'US'
     region_name = 'us-west-2'
     connectionCls = EC2USWestOregonConnection
-    _instance_types = EC2_US_WEST_OREGON_INSTANCE_TYPES
 
 
 class EC2APSEConnection(EC2Connection):
     """
-    Connection class for EC2 in the Southeast Asia Pacific Region
+    Connection class for EC2 in the Southeast Asia Pacific Region.
     """
 
-    host = EC2_AP_SOUTHEAST_HOST
+    host = REGION_DETAILS['ap-southeast-1']['endpoint']
 
 
 class EC2APNEConnection(EC2Connection):
     """
-    Connection class for EC2 in the Northeast Asia Pacific Region
+    Connection class for EC2 in the Northeast Asia Pacific Region.
     """
 
-    host = EC2_AP_NORTHEAST_HOST
+    host = REGION_DETAILS['ap-northeast-1']['endpoint']
 
 
 class EC2APSENodeDriver(EC2NodeDriver):
     """
-    Driver class for EC2 in the Southeast Asia Pacific Region
+    Driver class for EC2 in the Southeast Asia Pacific Region.
     """
 
     api_name = 'ec2_ap_southeast'
@@ -1344,12 +1511,11 @@ class EC2APSENodeDriver(EC2NodeDriver):
     country = 'SG'
     region_name = 'ap-southeast-1'
     connectionCls = EC2APSEConnection
-    _instance_types = EC2_AP_SOUTHEAST_INSTANCE_TYPES
 
 
 class EC2APNENodeDriver(EC2NodeDriver):
     """
-    Driver class for EC2 in the Northeast Asia Pacific Region
+    Driver class for EC2 in the Northeast Asia Pacific Region.
     """
 
     api_name = 'ec2_ap_northeast'
@@ -1358,20 +1524,19 @@ class EC2APNENodeDriver(EC2NodeDriver):
     country = 'JP'
     region_name = 'ap-northeast-1'
     connectionCls = EC2APNEConnection
-    _instance_types = EC2_AP_NORTHEAST_INSTANCE_TYPES
 
 
 class EC2SAEastConnection(EC2Connection):
     """
-    Connection class for EC2 in the South America (Sao Paulo) Region
+    Connection class for EC2 in the South America (Sao Paulo) Region.
     """
 
-    host = EC2_SA_EAST_HOST
+    host = REGION_DETAILS['sa-east-1']['endpoint']
 
 
 class EC2SAEastNodeDriver(EC2NodeDriver):
     """
-    Driver class for EC2 in the South America (Sao Paulo) Region
+    Driver class for EC2 in the South America (Sao Paulo) Region.
     """
 
     api_name = 'ec2_sa_east'
@@ -1380,7 +1545,27 @@ class EC2SAEastNodeDriver(EC2NodeDriver):
     country = 'BR'
     region_name = 'sa-east-1'
     connectionCls = EC2SAEastConnection
-    _instance_types = EC2_SA_EAST_INSTANCE_TYPES
+
+
+class EC2APSESydneyConnection(EC2Connection):
+    """
+    Connection class for EC2 in the Southeast Asia Pacific (Sydney) Region.
+    """
+
+    host = REGION_DETAILS['ap-southeast-2']['endpoint']
+
+
+class EC2APSESydneyNodeDriver(EC2NodeDriver):
+    """
+    Driver class for EC2 in the Southeast Asia Pacific (Sydney) Region.
+    """
+
+    api_name = 'ec2_ap_southeast_2'
+    name = 'Amazon EC2 (ap-southeast-2)'
+    friendly_name = 'Amazon Asia-Pacific Sydney'
+    country = 'AU'
+    region_name = 'ap-southeast-2'
+    connectionCls = EC2APSESydneyConnection
 
 
 class EucConnection(EC2Connection):
@@ -1391,14 +1576,16 @@ class EucConnection(EC2Connection):
     host = None
 
 
-class EucNodeDriver(EC2NodeDriver):
+class EucNodeDriver(BaseEC2NodeDriver):
     """
     Driver class for Eucalyptus
     """
 
     name = 'Eucalyptus'
+    website = 'http://www.eucalyptus.com/'
+    api_name = 'ec2_us_east'
+    region_name = 'us-east-1'
     connectionCls = EucConnection
-    _instance_types = EC2_US_WEST_INSTANCE_TYPES
 
     def __init__(self, key, secret=None, secure=True, host=None,
                  path=None, port=None):
@@ -1424,32 +1611,6 @@ class EucNodeDriver(EC2NodeDriver):
         """
         pass
 
-# Nimbus clouds have 3 EC2-style instance types but their particular RAM
-# allocations are configured by the admin
-NIMBUS_INSTANCE_TYPES = {
-    'm1.small': {
-        'id': 'm1.small',
-        'name': 'Small Instance',
-        'ram': None,
-        'disk': None,
-        'bandwidth': None,
-    },
-    'm1.large': {
-        'id': 'm1.large',
-        'name': 'Large Instance',
-        'ram': None,
-        'disk': None,
-        'bandwidth': None,
-    },
-    'm1.xlarge': {
-        'id': 'm1.xlarge',
-        'name': 'Extra Large Instance',
-        'ram': None,
-        'disk': None,
-        'bandwidth': None,
-    },
-}
-
 
 class NimbusConnection(EC2Connection):
     """
@@ -1459,22 +1620,23 @@ class NimbusConnection(EC2Connection):
     host = None
 
 
-class NimbusNodeDriver(EC2NodeDriver):
+class NimbusNodeDriver(BaseEC2NodeDriver):
     """
     Driver class for Nimbus
     """
 
     type = Provider.NIMBUS
     name = 'Nimbus'
+    website = 'http://www.nimbusproject.org/'
+    country = 'Private'
     api_name = 'nimbus'
     region_name = 'nimbus'
     friendly_name = 'Nimbus Private Cloud'
     connectionCls = NimbusConnection
-    _instance_types = NIMBUS_INSTANCE_TYPES
 
     def ex_describe_addresses(self, nodes):
         """
-        Nimbus doesn't support elastic IPs, so this is a passthrough
+        Nimbus doesn't support elastic IPs, so this is a passthrough.
 
         @inherits: L{EC2NodeDriver.ex_describe_addresses}
         """
@@ -1486,7 +1648,7 @@ class NimbusNodeDriver(EC2NodeDriver):
 
     def ex_create_tags(self, resource, tags):
         """
-        Nimbus doesn't support creating tags, so this is a passthrough
+        Nimbus doesn't support creating tags, so this is a passthrough.
 
         @inherits: L{EC2NodeDriver.ex_create_tags}
         """
