@@ -161,7 +161,7 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(len(ret), 1)
         node = ret[0]
         self.assertEqual('67.23.21.33', node.public_ips[0])
-        self.assertEqual('10.176.168.218', node.private_ips[0])
+        self.assertTrue('10.176.168.218' in node.private_ips)
         self.assertEqual(node.extra.get('flavorId'), '1')
         self.assertEqual(node.extra.get('imageId'), '11')
         self.assertEqual(type(node.extra.get('metadata')), type(dict()))
@@ -654,8 +654,9 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual('12065', node.id)
         self.assertEqual('50.57.94.35', node.public_ips[0])
         self.assertEqual('2001:4801:7808:52:16:3eff:fe47:788a', node.public_ips[1])
-        self.assertEqual('10.182.64.34', node.private_ips[0])
-        self.assertEqual('fec0:4801:7808:52:16:3eff:fe60:187d', node.private_ips[1])
+        self.assertTrue('10.182.64.34' in  node.private_ips)
+        self.assertTrue('12.16.18.28' in node.private_ips)
+        self.assertTrue('fec0:4801:7808:52:16:3eff:fe60:187d' in node.private_ips)
 
         self.assertEqual(node.extra.get('flavorId'), '2')
         self.assertEqual(node.extra.get('imageId'), '7')
